# Routing

[react-router-dom](https://www.npmjs.com/package/react-router) is used for routing in React apps. It's the standard routing library. React-router keeps your UI in sync with the URL.

### Getting Started

Import [setup](setup/) project into your workspace. Setup project contains the initial template you should start working on.
You can find the completed version of this project in [solution](solution/) folder.

Before starting, run `npm i`, which will download the router packages defined in `package.json`.

### Creating different pages

The first step is to create some pages in order to navigate between different pages.
Pages are stored under `containers` directory. Create a folder names `containers` inside `src`, and add the following pages inside:

```
containers
├── About
│   └── index.tsx
├── Contact
│   └── index.tsx
├── Home
│   └── index.tsx
└── NotFound.tsx
```

#### About/index.tsx

Add the following content to `About` page

```
import Heading from '../../components/Heading'

function About() {
    return <Heading title="About" />;
}

export default About;
```

#### Contact/index.tsx

Add the following content to `Contact` page

```
import Heading from "../../components/Heading";

function Contact() {
    return <Heading title="Contact" />;
}

export default Contact;
```

#### Home/index.tsx

Copy the content of `App.tsx` to `Home` page. `App.tsx` will handle route definition and changes.

```
import { useState } from 'react';
import Heading from '../../components/Heading'
import Table from '../../components/Table'
import Form from '../../components/Form'
import { default as data } from '../../data.json';

const Home = () => {
	const [productList, setProductList] = useState(data.productList);

	const onDelete = (id: string) => {
		const newList = productList.filter(item => item.id !== id)

		setProductList(newList);
	};

	const onAdd = (product: any) => {
    const newList = [...productList, product];

		setProductList(newList);
	};

	return (
		<div className="home-container">
			<Heading title="Products" />
			<Table data={productList} onDelete={onDelete} />
			<Form onSubmit={onAdd}/>
		</div>
	);
};

export default Home;
```

#### NotFound.tsx

* Finally, create a `NotFound` page, which is among routes and will be navigated by the paths other than the defined ones.

```
import Heading from './../components/Heading';

export default function NotFound() {
    return (
        <div>
            <Heading title="Not Found" />
            <p className="text-center">The link you requested is not found on this server. Please try a link from the menu.</p>
        </div>
    );
}
```

### Integrating react-router-dom

* Go to project folder and open terminal with git bash.
* Install "react-router-dom" inside the project from the terminal with "npm i react-router-dom". This installs latest version of the package.
* Add the following imports to `App.tsx`

```
import {
  Route,
  Routes,
  Link,
  Outlet
} from "react-router-dom";
import './App.css';
import About from "./containers/About";
import Home from "./containers/Home";
import Contact from './containers/Contact';
import NotFound from "./containers/NotFound";
```

* Routes are defined with the `Route` component:

`<Route path="about" element={<About />} />`

* In order to navigate to a route, `Link` component is used: 

`<Link to="/about">About</Link>`

* Below is the final content of `App.tsx`, where there is a navigation menu and routes together:

```
import {
  Route,
  Routes,
  Link,
  Outlet
} from "react-router-dom";
import './App.css';
import About from "./containers/About";
import Home from "./containers/Home";
import Contact from './containers/Contact';
import NotFound from "./containers/NotFound";

function App() {
  return (
    <div className="App">
        <Routes>
        <Route path="/" element={<Layout />}>
          <Route index element={<Home />} />
          <Route path="about" element={<About />} />
          <Route path="contact" element={<Contact />} />
          <Route path="*" element={<NotFound />} />
        </Route>
      </Routes>

    </div>
  );
}

function Layout() {
  return (
    <div>
      {/* A "layout route" is a good place to put markup you want to
          share across all the pages on your site, like navigation. */}
      <nav className="menu">
        <ul>
          <li>
            <Link to="/">Home</Link>
          </li>
          <li>
            <Link to="/about">About</Link>
          </li>
          <li>
            <Link to="/contact">Dashboard</Link>
          </li>
        </ul>
      </nav>

      {/* An <Outlet> renders whatever child route is currently active,
          so you can think about this <Outlet> as a placeholder for
          the child routes we defined above. */}
      <Outlet />
    </div>
  );
}

export default App;
```

* Finally, update `index.tsx` like this;

```js
import { BrowserRouter } from 'react-router-dom';

// ... other codes

root.render(
  <React.StrictMode>
    <BrowserRouter> // Add this line
      <App />
    </BrowserRouter>
  </React.StrictMode>
);
```

Now, you are ready to use your new navigation.

